@@ -18,7 +18,6 @@
 #' * `median_desc` sorts the x-axis by descending median of y-values.
 #' * `median` is an alias for `median_asc`.
 #' @param flip A logical value to flip the plot.
-#' @param keep_empty A logical value to keep the empty levels in the x-axis.
 #' @param group_by A character string of the column name to dodge the boxes/violins
 #' @param group_by_sep A character string to concatenate the columns in `group_by`, if multiple columns are provided.
 #' @param group_name A character string to name the legend of dodge.
@@ -122,7 +121,7 @@
 BoxViolinPlotAtomic <- function(
     data, x, x_sep = "_", y = NULL, base = c("box", "violin", "none"), in_form = c("long", "wide"),
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, keep_empty = FALSE, keep_na = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     paired_by = NULL, x_text_angle = ifelse(isTRUE(flip) && isTRUE(stack), 90, 45), step_increase = 0.1,
     fill_mode = ifelse(!is.null(group_by), "dodge", "x"), fill_reverse = FALSE, symnum_args = NULL,
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
@@ -165,6 +164,15 @@ BoxViolinPlotAtomic <- function(
     base_size <- theme_args$base_size %||% 12
     sig_labelsize <- sig_labelsize * base_size / 12
 
+    data <- process_keep_na_empty(data, keep_na, keep_empty)
+    keep_empty_x <- keep_empty[[x]]
+    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
+    keep_empty_facet <- if (!is.null(facet_by)) keep_empty[[facet_by[1]]] else NULL
+    if (length(facet_by) > 1) {
+        stopifnot("[Box/Violin/BeeswarmPlot] `keep_empty` for `facet_by` variables must be identical." =
+            identical(keep_empty_facet, keep_empty[[facet_by[2]]]))
+    }
+
     # Validate beeswarm parameters
     if (isTRUE(add_beeswarm)) {
         if (!requireNamespace("ggbeeswarm", quietly = TRUE)) {
@@ -266,11 +274,18 @@ BoxViolinPlotAtomic <- function(
     if (sort_x != "none" && !is.null(facet_by)) {
         stop("Cannot sort x-axis when facet_by is provided.")
     }
+    orig_data <- data
     data <- data %>%
         dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
         mutate(.y_mean = mean(!!sym(y)), .y_median = median(!!sym(y))) %>%
         ungroup()
 
+    # keep the factor levels
+    for (col in unique(c(x, group_by, facet_by))) {
+        data[[col]] <- factor(data[[col]], levels = levels(orig_data[[col]]))
+    }
+    rm(orig_data)
+
     values <- data[[y]][is.finite(data[[y]])]
     if (is.character(y_max)) {
         q_max <- as.numeric(sub("(^q)(\\d+)", "\\2", y_max)) / 100
@@ -343,12 +358,13 @@ BoxViolinPlotAtomic <- function(
     }
     p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y), fill = !!sym(fill_by)))
     if (isTRUE(add_bg)) {
-        p <- p + bg_layer(data, x, bg_palette, bg_palcolor, bg_alpha, keep_empty, facet_by)
+        p <- p + bg_layer(data, x, isTRUE(keep_empty_x), bg_palette, bg_palcolor, bg_alpha, facet_by)
     }
 
     if (base == "box" || (base == "none" && isTRUE(add_box))) {
         p <- p + geom_boxplot(
-            position = position_dodge(width = 0.9), color = "black", width = 0.8, outlier.shape = NA
+            position = position_dodge(width = 0.9), color = "black",
+            width = 0.8, outlier.shape = NA, show.legend = TRUE
         )
     } else if (base == "violin") {
         p <- p + geom_violin(
@@ -356,19 +372,43 @@ BoxViolinPlotAtomic <- function(
             # See https://github.com/tidyverse/ggplot2/issues/2801
             # There is a fix but not yet released
             position = position_dodge(width = 0.9), scale = "width", trim = TRUE,
-            alpha = alpha, width = 0.8
+            alpha = alpha, width = 0.8, show.legend = TRUE
         )
     }
     if (fill_mode == "dodge") {
-        p <- p + scale_fill_manual(
-            name = group_name %||% group_by,
-            values = palette_this(levels(data[[group_by]]), palette = palette, palcolor = palcolor)
-        )
+        group_vals <- levels(data[[group_by]])
+        if (anyNA(group_vals)) group_vals <- c(group_vals, NA)
+        group_colors <- palette_this(group_vals, palette = palette, palcolor = palcolor, NA_keep = TRUE)
+
+        if (isTRUE(keep_empty_group)) {
+            p <- p + scale_fill_manual(
+                name = group_name %||% group_by,
+                values = group_colors, na.value = group_colors['NA'] %||% "grey80",
+                breaks = group_vals, limits = group_vals, drop = FALSE
+            )
+        } else {
+            p <- p + scale_fill_manual(
+                name = group_name %||% group_by,
+                values = group_colors, na.value = group_colors['NA'] %||% "grey80"
+            )
+        }
     } else if (fill_mode == "x") {
-        p <- p + scale_fill_manual(
-            name = x,
-            values = palette_this(levels(data[[x]]), palette = palette, palcolor = palcolor)
-        )
+        x_vals <- levels(data[[x]])
+        if (anyNA(x_vals)) x_vals <- c(x_vals, NA)
+        x_colors <- palette_this(x_vals, palette = palette, palcolor = palcolor, NA_keep = TRUE)
+
+        if (isTRUE(keep_empty_x)) {
+            p <- p + scale_fill_manual(
+                name = x,
+                values = x_colors, na.value = x_colors['NA'] %||% "grey80",
+                breaks = x_vals, limits = x_vals, drop = FALSE
+            )
+        } else {
+            p <- p + scale_fill_manual(
+                name = x,
+                values = x_colors, na.value = x_colors['NA'] %||% "grey80"
+            )
+        }
     } else {
         p <- p + scale_fill_gradientn(
             name = paste0(y, " (", fill_mode, ")"),
@@ -815,10 +855,20 @@ BoxViolinPlotAtomic <- function(
                 position = position_dodge(width = 0.9), linewidth = trend_linewidth
             )
             if (!is.null(group_by)) {
-                p <- p + scale_color_manual(
-                    values = palette_this(levels(data[[group_by]]), palette = palette, palcolor = palcolor),
-                    guide = "none"
-                )
+                group_vals <- levels(data[[group_by]])
+                if (anyNA(group_vals)) group_vals <- c(group_vals, NA)
+                group_colors <- palette_this(group_vals, palette = palette, palcolor = palcolor, NA_keep = TRUE)
+
+                if (isTRUE(keep_empty_group)) {
+                    p <- p + scale_color_manual(
+                        values = group_colors, na.value = group_colors['NA'] %||% "grey80",
+                        breaks = group_vals, limits = group_vals, drop = FALSE
+                    )
+                } else {
+                    p <- p + scale_color_manual(
+                        values = group_colors, na.value = group_colors['NA'] %||% "grey80"
+                    )
+                }
             }
         } else {
             p <- p + stat_summary(
@@ -865,7 +915,7 @@ BoxViolinPlotAtomic <- function(
 
     just <- calc_just(x_text_angle)
     p <- p +
-        scale_x_discrete(drop = !keep_empty) +
+        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
         labs(title = title, subtitle = subtitle, x = xlab %||% x, y = ylab %||% y)
 
     p <- p + scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks)
@@ -956,7 +1006,7 @@ BoxViolinPlotAtomic <- function(
 
     facet_plot(p, facet_by, facet_scales, facet_nrow, facet_ncol, facet_byrow,
         strip.position = strip_position, legend.position = legend.position,
-        legend.direction = legend.direction
+        legend.direction = legend.direction, drop = !isTRUE(keep_empty_facet)
     )
 }
 
@@ -972,7 +1022,7 @@ BoxViolinPlot <- function(
     data, x, x_sep = "_", y = NULL, base = c("box", "violin"), in_form = c("long", "wide"),
     split_by = NULL, split_by_sep = "_", symnum_args = NULL,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, keep_empty = FALSE, keep_na = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     paired_by = NULL, x_text_angle = ifelse(isTRUE(flip) && isTRUE(stack), 90, 45), step_increase = 0.1,
     fill_mode = ifelse(!is.null(group_by), "dodge", "x"), fill_reverse = FALSE,
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
@@ -994,10 +1044,16 @@ BoxViolinPlot <- function(
     combine = TRUE, nrow = NULL, ncol = NULL, byrow = TRUE,
     axes = NULL, axis_titles = axes, guides = NULL, design = NULL, ...) {
     validate_common_args(seed)
+    keep_na <- check_keep_na(keep_na, c(x, split_by, group_by, paired_by, facet_by))
+    keep_empty <- check_keep_empty(keep_empty, c(x, split_by, group_by, paired_by, facet_by))
     theme <- process_theme(theme)
     split_by <- check_columns(data, split_by, force_factor = TRUE, allow_multi = TRUE, concat_multi = TRUE, concat_sep = split_by_sep)
 
     if (!is.null(split_by)) {
+        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
+        keep_na[[split_by]] <- NULL
+        keep_empty[[split_by]] <- NULL
+
         datas <- split(data, data[[split_by]])
         # keep the order of levels
         datas <- datas[levels(data[[split_by]])]
@@ -1023,7 +1079,7 @@ BoxViolinPlot <- function(
             }
             BoxViolinPlotAtomic(datas[[nm]],
                 x = x, x_sep = x_sep, y = y, base = base, in_form = in_form,
-                sort_x = sort_x, flip = flip, keep_empty = keep_empty, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
+                sort_x = sort_x, flip = flip, keep_empty = keep_empty, keep_na = keep_na, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
                 paired_by = paired_by, x_text_angle = x_text_angle, fill_mode = fill_mode, fill_reverse = fill_reverse, step_increase = step_increase,
                 theme = theme, theme_args = theme_args, palette = palette[[nm]], palcolor = palcolor[[nm]], alpha = alpha,
                 aspect.ratio = aspect.ratio, legend.position = legend.position[[nm]], legend.direction = legend.direction[[nm]],
@@ -1115,12 +1171,49 @@ BoxViolinPlot <- function(
 #'     paired_by = "subject", group_by = "group",
 #'     comparisons = TRUE, pt_size = 3, pt_color = "red"
 #' )
+#'
+#' # keep_na and keep_empty example
+#' data <- data.frame(
+#'     x = factor(rep(c(LETTERS[1:3], NA, LETTERS[5:8]), each = 40),
+#'        levels = c(LETTERS[1:8])),
+#'     y = c(rnorm(160), rnorm(160, mean = 1)),
+#'     group1 = sample(c("g1", "g2"), 320, replace = TRUE),
+#'     group2 = factor(sample(c("h1", NA, "h3", "h4"), 320, replace = TRUE),
+#'        levels = c("h1", "h2", "h3", "h4"))
+#' )
+#'
+#' BoxPlot(data, x = "x", y = "y",
+#'     title = "keep_na = FALSE; keep_empty = FALSE")
+#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE,
+#'     title = "keep_na = TRUE; keep_empty = TRUE")
+#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE,
+#'     title = "keep_na = TRUE; keep_empty = TRUE", facet_by = "group2")
+#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = 'level',
+#'     title = "keep_na = TRUE; keep_empty = 'level'")
+#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
+#'     title = "keep_na = FALSE; keep_empty = FALSE; group_by = 'group2'")
+#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
+#'     keep_na = TRUE, keep_empty = TRUE,
+#'     title = "keep_na = TRUE; keep_empty = TRUE; group_by = 'group2'")
+#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
+#'     keep_na = TRUE, keep_empty = 'level',
+#'     title = "keep_na = TRUE; keep_empty = 'level'; group_by = 'group2'")
+#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
+#'     keep_na = list(x = TRUE, group2 = FALSE),
+#'     keep_empty = list(x = FALSE, group2 = TRUE),
+#'     title = "keep_na: x=TRUE, group2=FALSE\nkeep_empty: x=FALSE, group2=TRUE"
+#' )
+#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
+#'     keep_na = list(x = FALSE, group2 = TRUE),
+#'     keep_empty = list(x = TRUE, group2 = FALSE),
+#'     title = "keep_na: x=FALSE, group2=TRUE\nkeep_empty: x=TRUE, group2=FALSE"
+#' )
 #' }
 BoxPlot <- function(
     data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"),
     split_by = NULL, split_by_sep = "_", symnum_args = NULL,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, keep_empty = FALSE, keep_na = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     paired_by = NULL, x_text_angle = ifelse(isTRUE(flip) && isTRUE(stack), 90, 45), step_increase = 0.1,
     fill_mode = ifelse(!is.null(group_by), "dodge", "x"), fill_reverse = FALSE,
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
@@ -1144,7 +1237,7 @@ BoxPlot <- function(
     BoxViolinPlot(
         data = data, x = x, x_sep = x_sep, y = y, base = "box", in_form = in_form,
         split_by = split_by, split_by_sep = split_by_sep,
-        sort_x = sort_x, flip = flip, keep_empty = keep_empty, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
+        sort_x = sort_x, flip = flip, keep_empty = keep_empty, keep_na = keep_na, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
         paired_by = paired_by, x_text_angle = x_text_angle, fill_mode = fill_mode, fill_reverse = fill_reverse, step_increase = step_increase,
         theme = theme, theme_args = theme_args, palette = palette, palcolor = palcolor, alpha = alpha,
         aspect.ratio = aspect.ratio, legend.position = legend.position, legend.direction = legend.direction,
@@ -1211,7 +1304,7 @@ ViolinPlot <- function(
     data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"),
     split_by = NULL, split_by_sep = "_", symnum_args = NULL,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, keep_empty = FALSE, keep_na = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     paired_by = NULL, x_text_angle = ifelse(isTRUE(flip) && isTRUE(stack), 90, 45), step_increase = 0.1,
     fill_mode = ifelse(!is.null(group_by), "dodge", "x"), fill_reverse = FALSE,
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
@@ -1236,7 +1329,7 @@ ViolinPlot <- function(
     BoxViolinPlot(
         data = data, x = x, x_sep = x_sep, y = y, base = "violin", in_form = in_form,
         split_by = split_by, split_by_sep = split_by_sep,
-        sort_x = sort_x, flip = flip, keep_empty = keep_empty, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
+        sort_x = sort_x, flip = flip, keep_empty = keep_empty, keep_na = keep_na, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
         paired_by = paired_by, x_text_angle = x_text_angle, fill_mode = fill_mode, fill_reverse = fill_reverse, step_increase = step_increase,
         theme = theme, theme_args = theme_args, palette = palette, palcolor = palcolor, alpha = alpha,
         aspect.ratio = aspect.ratio, legend.position = legend.position, legend.direction = legend.direction,
@@ -1286,7 +1379,7 @@ BeeswarmPlot <- function(
     data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"),
     split_by = NULL, split_by_sep = "_", symnum_args = NULL,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, keep_empty = FALSE, keep_na = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     paired_by = NULL, x_text_angle = ifelse(isTRUE(flip) && isTRUE(stack), 90, 45), step_increase = 0.1,
     fill_mode = ifelse(!is.null(group_by), "dodge", "x"), fill_reverse = FALSE,
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
@@ -1314,7 +1407,7 @@ BeeswarmPlot <- function(
     BoxViolinPlot(
         data = data, x = x, x_sep = x_sep, y = y, base = "none", in_form = in_form,
         split_by = split_by, split_by_sep = split_by_sep,
-        sort_x = sort_x, flip = flip, keep_empty = keep_empty, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
+        sort_x = sort_x, flip = flip, keep_empty = keep_empty, keep_na = keep_na, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
         paired_by = paired_by, x_text_angle = x_text_angle, fill_mode = fill_mode, fill_reverse = fill_reverse, step_increase = step_increase,
         theme = theme, theme_args = theme_args, palette = palette, palcolor = palcolor, alpha = alpha,
         aspect.ratio = aspect.ratio, legend.position = legend.position, legend.direction = legend.direction,
