@@ -41,12 +41,14 @@ prepare_fgsea_result <- function(data) {
     data$pvalue <- data$pval
     data$pval <- NULL
     data$p.adjust <- data$padj
+    data$p.adjust[is.na(data$p.adjust)] <- 1
     data$padj <- NULL
     if (is.character(data$leadingEdge)) {
         data$leadingEdge <- strsplit(data$leadingEdge, ",")
     }
     data$core_enrichment <- sapply(data$leadingEdge, paste0, collapse = "/")
     data$leadingEdge <- NULL
+    data$NES[is.na(data$NES)] <- 0
 
     data
 }
