@@ -17,7 +17,6 @@
 #' * `median_desc` sorts the x-axis by descending median of y-values.
 #' * `median` is an alias for `median_asc`.
 #' @param flip A logical value to flip the plot.
-#' @param keep_empty A logical value to keep the empty levels in the x-axis.
 #' @param group_by A character string to dodge the points.
 #' @param group_by_sep A character string to concatenate the columns in `group_by`, if multiple columns are provided.
 #' @param group_name A character string to name the legend of dodge.
@@ -60,9 +59,9 @@
 #' @importFrom ggplot2 position_jitterdodge scale_fill_manual scale_color_manual scale_y_continuous scale_size_area guide_legend guide_colorbar
 #' @importFrom ggrepel geom_text_repel
 JitterPlotAtomic <- function(
-    data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"),
+    data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"), keep_na = FALSE, keep_empty = FALSE,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     x_text_angle = 0, order_by = "-({y}^2 + {size_by}^2)",
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
     aspect.ratio = NULL, legend.position = "right", legend.direction = "vertical",
@@ -89,12 +88,26 @@ JitterPlotAtomic <- function(
     group_by <- check_columns(data, group_by, force_factor = TRUE, allow_multi = TRUE, concat_multi = TRUE, concat_sep = group_by_sep)
     facet_by <- check_columns(data, facet_by, force_factor = TRUE, allow_multi = TRUE)
 
+    data <- process_keep_na_empty(data, keep_na, keep_empty)
+    keep_empty_x <- keep_empty[[x]]
+    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
+    keep_empty_facet <- if (!is.null(facet_by)) keep_empty[[facet_by[1]]] else NULL
+    if (length(facet_by) > 1) {
+        stopifnot("[JitterPlot] `keep_empty` for `facet_by` variables must be identical." =
+            identical(keep_empty_facet, keep_empty[[facet_by[2]]]))
+    }
+
     sort_x <- match.arg(sort_x)
     data <- data %>%
         dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
         mutate(.y_mean = mean(!!sym(y)), .y_median = median(!!sym(y))) %>%
         ungroup()
 
+    # keep factor levels
+    for (col in unique(c(x, group_by, facet_by))) {
+        data[[col]] <- factor(data[[col]], levels = levels(data[[col]]))
+    }
+
     values <- data[[y]][is.finite(data[[y]])]
     if (is.character(y_max)) {
         q_max <- as.numeric(sub("(^q)(\\d+)", "\\2", y_max)) / 100
@@ -147,6 +160,9 @@ JitterPlotAtomic <- function(
                 dplyr::mutate(.show_label = dplyr::row_number() <= nlabel) %>%
                 dplyr::ungroup()
         }
+        for (col in unique(c(x, facet_by))) {
+            data[[col]] <- factor(data[[col]], levels = levels(data[[col]]))
+        }
     }
     data <- as.data.frame(data)
 
@@ -166,10 +182,29 @@ JitterPlotAtomic <- function(
         if (length(aspect.ratio) == 0 || is.na(aspect.ratio)) aspect.ratio <- NULL
     }
 
+    color_col <- ifelse(is.null(group_by), x, group_by)
+    keep_empty_col <- if (color_col == x) keep_empty_x else keep_empty_group
+    col_levels <- levels(data[[color_col]])
+    if (anyNA(data[[color_col]])) col_levels <- c(col_levels, NA)
+    colors <- palette_this(col_levels, palette = palette, palcolor = palcolor, NA_keep = TRUE)
+    if (anyNA(col_levels)) {
+        # To prevent ggrepel text segments being too long
+        # It seem it thinks the NA is an empty area for it to place labels
+        names(colors)[is.na(names(colors))] <- "NA"
+        col_levels[is.na(col_levels)] <- "NA"
+        levels(data[[color_col]]) <- col_levels
+        data[[color_col]][is.na(data[[color_col]])] <- "NA"
+    }
+    # Similar when x has NA, but color_col is group
+    if (color_col != x && anyNA(data[[x]])) {
+        x_levels <- c(levels(data[[x]]), "NA")
+        levels(data[[x]]) <- x_levels
+        data[[x]][is.na(data[[x]])] <- "NA"
+    }
     # Base
     p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
     if (isTRUE(add_bg)) {
-        p <- p + bg_layer(data, x, bg_palette, bg_palcolor, bg_alpha, keep_empty, facet_by)
+        p <- p + bg_layer(data, x, isTRUE(keep_empty_x), bg_palette, bg_palcolor, bg_alpha, facet_by)
     }
 
     # Positioner (jitter + optional dodge)
@@ -178,6 +213,23 @@ JitterPlotAtomic <- function(
         dodge.width = ifelse(is.null(group_by), 0, 0.9), seed = seed
     )
 
+    # Pre-calculate jittered positions for labels (geom_text_repel doesn't respect position adjustments)
+    if (any(data$.show_label)) {
+        # Create a temporary plot to extract jittered positions
+        temp_data <- data
+        temp_mapping <- aes(x = !!sym(x), y = !!sym(y))
+        if (!is.null(group_by)) {
+            temp_mapping$group <- aes(group = !!sym(group_by))$group
+        }
+        temp_plot <- ggplot2::ggplot(temp_data, temp_mapping) +
+            geom_point(position = pos) +
+            scale_x_discrete(drop = !isTRUE(keep_empty_x))
+        temp_build <- ggplot2::ggplot_build(temp_plot)
+        jittered_coords <- temp_build$data[[1]]
+        data$.x_jittered <- jittered_coords$x
+        data$.y_jittered <- jittered_coords$y
+    }
+
     # Build point layer, color by x
     has_fill <- shape %in% 21:25
     point_args <- list(
@@ -186,10 +238,10 @@ JitterPlotAtomic <- function(
     mapping <- list(aes())
 
     if (has_fill) {
-        mapping[[length(mapping) + 1]] <- aes(fill = !!sym(x))
+        mapping[[length(mapping) + 1]] <- aes(fill = !!sym(color_col))
         # border handling
         if (isTRUE(border)) {
-            mapping[[length(mapping) + 1]] <- aes(color = !!sym(x))
+            mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_col))
         } else if (is.character(border) && length(border) == 1) {
             point_args$color <- border
         } else {
@@ -197,7 +249,7 @@ JitterPlotAtomic <- function(
         }
     } else {
         # shapes without fill
-        mapping[[length(mapping) + 1]] <- aes(color = !!sym(x))
+        mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_col))
     }
 
     # size handling
@@ -226,23 +278,31 @@ JitterPlotAtomic <- function(
     modify_list <- utils::getFromNamespace("modify_list", "ggplot2")
     point_args$mapping <- Reduce(modify_list, mapping)
     point_args$data <- data
+    point_args$show.legend <- TRUE
     p <- p + do.call(geom_point, point_args)
 
     # Discrete color/fill scales by x
-    x_levels <- levels(data[[x]])
     if (has_fill) {
-        p <- p + scale_fill_manual(
-            name = x, values = palette_this(x_levels, palette = palette, palcolor = palcolor), drop = !keep_empty
-        )
-        if (isTRUE(border)) {
-            p <- p + scale_color_manual(
-                values = palette_this(x_levels, palette = palette, palcolor = palcolor), guide = "none", drop = !keep_empty
+        if (isTRUE(keep_empty_col)) {
+            p <- p + scale_fill_manual(
+                name = color_col, values = colors, na.value = colors['NA'] %||% "grey80",
+                breaks = col_levels, limits = col_levels, drop = FALSE
             )
+        } else {
+            p <- p + scale_fill_manual(name = color_col, values = colors, na.value = colors['NA'] %||% "grey80")
+        }
+        if (isTRUE(border)) {
+            p <- p + scale_color_manual(values = colors, guide = "none", na.value = colors['NA'] %||% "grey80")
         }
     } else {
-        p <- p + scale_color_manual(
-            name = x, values = palette_this(x_levels, palette = palette, palcolor = palcolor), drop = !keep_empty
-        )
+        if (isTRUE(keep_empty_col)) {
+            p <- p + scale_color_manual(
+                name = color_col, values = colors, na.value = colors['NA'] %||% "grey80",
+                breaks = col_levels, limits = col_levels, drop = FALSE
+            )
+        } else {
+            p <- p + scale_color_manual(name = color_col, values = colors, na.value = colors['NA'] %||% "grey80")
+        }
     }
 
     # Size scale when mapped
@@ -297,7 +357,7 @@ JitterPlotAtomic <- function(
     if (any(data$.show_label)) {
         p <- p + geom_text_repel(
             data = data[data$.show_label, , drop = FALSE],
-            mapping = aes(label = !!sym(".label")),
+            mapping = aes(x = !!sym(".x_jittered"), y = !!sym(".y_jittered"), label = !!sym(".label")),
             color = label_fg, bg.color = label_bg, bg.r = label_bg_r,
             size = label_size, min.segment.length = 0, segment.color = "grey40",
             max.overlaps = 100
@@ -313,7 +373,7 @@ JitterPlotAtomic <- function(
 
     just <- calc_just(x_text_angle)
     p <- p +
-        scale_x_discrete(drop = !keep_empty) +
+        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
         scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks) +
         labs(title = title, subtitle = subtitle, x = xlab %||% x, y = ylab %||% y)
 
@@ -370,7 +430,7 @@ JitterPlotAtomic <- function(
 
     facet_plot(p, facet_by, facet_scales, facet_nrow, facet_ncol, facet_byrow,
         strip.position = strip_position, legend.position = legend.position,
-        legend.direction = legend.direction
+        legend.direction = legend.direction, drop = !isTRUE(keep_empty_facet)
     )
 }
 
@@ -389,15 +449,21 @@ JitterPlotAtomic <- function(
 #' @examples
 #' \donttest{
 #' set.seed(8525)
-#' n <- 200
-#' x <- sample(LETTERS[1:5], n, replace = TRUE)
-#' group <- sample(c("G1", "G2"), n, replace = TRUE)
+#' n <- 180
+#' x <- factor(
+#'     sample(c("A", NA, LETTERS[3:5]), n, replace = TRUE),
+#'     levels = c("A", "B", "C", "D", "E")
+#' )
+#' group <- factor(
+#'     sample(c("G1", NA, "G3"), n, replace = TRUE),
+#'     levels = c("G1", "G2", "G3")
+#' )
 #' size <- rexp(n, rate = 1)
 #' id <- paste0("pt", seq_len(n))
-#' y <- rnorm(n, mean = ifelse(group == "G1", 0.5, -0.5)) +
-#'      as.numeric(factor(x, levels = LETTERS[1:5]))/10
+#' y <- rnorm(n, mean = ifelse(is.na(group), 0, ifelse(group == "G1", 0.5, -0.5))) +
+#'      as.numeric(ifelse(is.na(x), 0, x))/10
 #' df <- data.frame(
-#'   x = factor(x, levels = LETTERS[1:5]),
+#'   x = x,
 #'   y = y,
 #'   group = group,
 #'   size = size,
@@ -407,6 +473,9 @@ JitterPlotAtomic <- function(
 #' # Basic
 #' JitterPlot(df, x = "x", y = "y")
 #'
+#' # Keep empty x levels and NA
+#' JitterPlot(df, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)
+#'
 #' # Map size with transform; legend shows original values
 #' JitterPlot(df, x = "x", y = "y", size_by = "size", size_name = "Abundance",
 #'     size_trans = sqrt, order_by = "-y^2")
@@ -415,6 +484,11 @@ JitterPlotAtomic <- function(
 #' JitterPlot(df, x = "x", y = "y", group_by = "group",
 #'   add_hline = 0, hline_type = "dashed", hline_color = "red2")
 #'
+#' # Keep the empty levels only for color coding
+#' # Note the G3 is not blue (which is taken by unused level G2)
+#' JitterPlot(df, x = "x", y = "y", group_by = "group",
+#'     keep_na = TRUE, keep_empty = 'level')
+#'
 #' # Label top points by distance (y^2 + size^2)
 #' JitterPlot(df, x = "x", y = "y", size_by = "size", label_by = "id", nlabel = 3)
 #'
@@ -423,9 +497,9 @@ JitterPlotAtomic <- function(
 #' }
 JitterPlot <- function(
     data, x, x_sep = "_", y = NULL, in_form = c("long", "wide"),
-    split_by = NULL, split_by_sep = "_",
+    split_by = NULL, split_by_sep = "_", keep_na = FALSE, keep_empty = FALSE,
     sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc", "median"),
-    flip = FALSE, keep_empty = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
+    flip = FALSE, group_by = NULL, group_by_sep = "_", group_name = NULL,
     x_text_angle = 0, order_by = "-({y}^2 + {size_by}^2)",
     theme = "theme_this", theme_args = list(), palette = "Paired", palcolor = NULL, alpha = 1,
     aspect.ratio = NULL, legend.position = "right", legend.direction = "vertical",
@@ -442,10 +516,15 @@ JitterPlot <- function(
     axes = NULL, axis_titles = axes, guides = NULL, design = NULL, ...
 ) {
     validate_common_args(seed)
+    keep_na <- check_keep_na(keep_na, c(x, split_by, group_by, facet_by))
+    keep_empty <- check_keep_empty(keep_empty, c(x, split_by, group_by, facet_by))
     theme <- process_theme(theme)
     split_by <- check_columns(data, split_by, force_factor = TRUE, allow_multi = TRUE, concat_multi = TRUE, concat_sep = split_by_sep)
 
     if (!is.null(split_by)) {
+        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
+        keep_na[[split_by]] <- NULL
+        keep_empty[[split_by]] <- NULL
         datas <- split(data, data[[split_by]])
         datas <- datas[levels(data[[split_by]])]
     } else {
@@ -467,8 +546,8 @@ JitterPlot <- function(
             }
             JitterPlotAtomic(
                 datas[[nm]],
-                x = x, x_sep = x_sep, y = y, in_form = in_form,
-                sort_x = sort_x, flip = flip, keep_empty = keep_empty, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
+                x = x, x_sep = x_sep, y = y, in_form = in_form, keep_na = keep_na, keep_empty = keep_empty,
+                sort_x = sort_x, flip = flip, group_by = group_by, group_by_sep = group_by_sep, group_name = group_name,
                 x_text_angle = x_text_angle, theme = theme, theme_args = theme_args, palette = palette[[nm]], palcolor = palcolor[[nm]], alpha = alpha,
                 aspect.ratio = aspect.ratio, legend.position = legend.position[[nm]], legend.direction = legend.direction[[nm]],
                 shape = shape, border = border, order_by = order_by,
